import require$$1 from 'path';
import { setArgs } from './args.mjs';
import { parseJSON } from '../util/parseJSON.mjs';
import { readFile } from '../util/readFile.mjs';
import { readPackageVersion } from '../util/readVersionPackage.mjs';

const defaultConfig = {
    snpCatalog: './.snp',
    template: 'node',
    sUpdaterVersion: undefined,
    templateVersion: undefined,
    fileMap: undefined,
    REPOSITORY_MAP_FILE_NAME: 'repositoryMap.json',
    projectCatalog: './',
    temporaryFolder: './.snp/temporary/',
    snpConfigFileName: 'snp.config.json',
    snpConfigFile: './.snp/snp.config.json',
    remoteRepository: 'https://raw.githubusercontent.com/SebastianWesolowski/s-update-manager/dev/template/',
    // remoteRepository: 'https://raw.githubusercontent.com/SebastianWesolowski/s-update-manager/main/template/',
    isDebug: false,
    _: [],
};
const createPath = function (parts, isFolder = false) {
    let joinedPath;
    let includeRoot = false;
    const rootRegex = /^\.\/.*$/;
    if (typeof parts === 'string') {
        joinedPath = parts;
        if (rootRegex.test(parts)) {
            includeRoot = true;
        }
    }
    else {
        if (rootRegex.test(parts.join(''))) {
            includeRoot = true;
        }
        joinedPath = require$$1.join(...parts);
    }
    if (isFolder) {
        joinedPath = require$$1.normalize(joinedPath + require$$1.sep);
    }
    if (includeRoot && joinedPath !== './') {
        joinedPath = './' + require$$1.normalize(joinedPath);
    }
    return joinedPath;
};
const regenerateConfig = async (config) => {
    const regeneratedConfig = { ...config };
    if (regeneratedConfig.projectCatalog) {
        regeneratedConfig.snpCatalog = createPath([regeneratedConfig.projectCatalog, '.snp/'], true);
        regeneratedConfig.temporaryFolder = createPath([regeneratedConfig.snpCatalog, 'temporary/'], true);
        regeneratedConfig.sUpdaterVersion = await readPackageVersion(createPath([regeneratedConfig.projectCatalog, 'package.json']));
        if (regeneratedConfig.snpConfigFileName) {
            regeneratedConfig.snpConfigFile = createPath([regeneratedConfig.snpCatalog, regeneratedConfig.snpConfigFileName]);
        }
    }
    return regeneratedConfig;
};
const updateConfig = async (config, keyToUpdate) => {
    const keyName = Object.keys(keyToUpdate)[0];
    const folderKey = ['snpCatalog', 'projectCatalog', 'temporaryFolder'];
    const fileKey = ['snpConfigFileName', 'snpConfigFile'];
    const isFolder = folderKey.includes(keyName);
    const isFile = fileKey.includes(keyName);
    let value = keyToUpdate[keyName];
    if (isFolder || isFile) {
        value = createPath(value, isFolder);
    }
    const valueToUpdate = { [keyName]: value };
    const updatedConfig = { ...config, ...valueToUpdate };
    return await regenerateConfig(updatedConfig);
};
const getConfig = async (args) => {
    let config = { ...defaultConfig };
    let localConfigFile = {};
    const argsObject = setArgs(args);
    config = await updateConfig(config, { isDebug: argsObject.isDebug || config.isDebug });
    config = await updateConfig(config, { projectCatalog: argsObject.projectCatalog || config.projectCatalog });
    config = await updateConfig(config, { snpCatalog: argsObject.snpCatalog || config.snpCatalog });
    config = await updateConfig(config, { snpConfigFile: argsObject.snpConfigFile || config.snpConfigFile });
    config = await updateConfig(config, { snpConfigFileName: argsObject.snpConfigFileName || config.snpConfigFileName });
    const dataLocalConfigFile = parseJSON(await readFile(config.snpConfigFile));
    if (dataLocalConfigFile !== '' && typeof dataLocalConfigFile === 'object') {
        localConfigFile = dataLocalConfigFile;
        config = { ...config, ...localConfigFile };
    }
    config = await updateConfig(config, { isDebug: argsObject.isDebug || localConfigFile.isDebug || config.isDebug });
    config = await updateConfig(config, {
        projectCatalog: argsObject.projectCatalog || localConfigFile.projectCatalog || config.projectCatalog,
    });
    config = await updateConfig(config, {
        snpCatalog: argsObject.snpCatalog || localConfigFile.snpCatalog || config.snpCatalog,
    });
    config = await updateConfig(config, {
        snpConfigFile: argsObject.snpConfigFile || localConfigFile.snpConfigFile || config.snpConfigFile,
    });
    config = await updateConfig(config, {
        snpConfigFileName: argsObject.snpConfigFileName || localConfigFile.snpConfigFileName || config.snpConfigFileName,
    });
    config = await updateConfig(config, {
        remoteRepository: argsObject.remoteRepository || localConfigFile.remoteRepository || config.remoteRepository,
    });
    config = await updateConfig(config, { template: argsObject.template || localConfigFile.template || config.template });
    return regenerateConfig(config);
};

export { createPath, getConfig };
