#!/usr/bin/env node
import minimist from '../node_modules/minimist/index.mjs';
import { buildFromConfig } from './feature/buildFromConfig.mjs';
import { cleanUp } from './feature/cleanUp.mjs';
import { getConfig } from './feature/defaultConfig.mjs';
import { updateFromRemote } from './feature/updateFromRemote.mjs';
import { createCatalog } from './util/createCatalog.mjs';
import { createFile } from './util/createFile.mjs';
import { debugFunction } from './util/debugFunction.mjs';
import { isFolderExist } from './util/isFolderExist.mjs';

//
// type PackageConfig = {
//   instructions: string;
//   default: string;
//   extends: string;
//   custom: string;
// };
//
// type Config = Record<
//   string,
//   {
//     name: string;
//     filePackage: PackageConfig;
//     sequence: string[];
//   }
// >;
//
// const config: Config = {
//   'README.md': {
//     name: 'README.md',
//     filePackage: {
//       instructions: 'README.md-instructions.md',
//       default: 'README.md-default.md',
//       extends: 'README.md-extends.md',
//       custom: 'README.md-custom.md',
//     },
//     sequence: ['default', 'extends', 'custom'],
//   },
// };
const init = async (args) => {
    const config = await getConfig(args);
    if (config.fileMap) {
        throw new Error('Config file exists, use build script or update');
    }
    debugFunction(config.isDebug, '=== Start SNP INIT ===', '[INIT]');
    return await createCatalog(config.temporaryFolder).then(() => {
        return { ...config };
    });
};
const createConfigFile = async (config) => {
    debugFunction(config.isDebug, { config }, '[INIT] debugFunction');
    const { snpCatalog, template, sUpdaterVersion } = config;
    await isFolderExist({
        folderPath: snpCatalog,
        createFolder: true,
    });
    await createFile({
        filePath: config.snpConfigFile,
        content: JSON.stringify(config),
        isDebug: config.isDebug,
    });
    debugFunction(config.isDebug, { sUpdaterVersion, template }, '[INIT] created snp config file');
    return config;
};
const args = minimist(process.argv.slice(2));
let finalConfig = {
    isDebug: false,
};
init(args)
    .then((config) => {
    finalConfig = config;
    return createConfigFile(config);
})
    .then((config) => {
    finalConfig = config;
    return updateFromRemote(config);
})
    .then((config) => {
    finalConfig = config;
    return buildFromConfig(config);
})
    .then((config) => {
    finalConfig = config;
    return cleanUp(config);
})
    .finally(() => {
    debugFunction(finalConfig?.isDebug, { finalConfig }, '[INIT] final config');
    debugFunction(finalConfig?.isDebug, '=== final SNP INIT ===');
});

export { createConfigFile, init };
