#!/usr/bin/env node
import minimist from '../node_modules/minimist/index.mjs';
import { buildFromConfig } from './feature/buildFromConfig.mjs';
import { cleanUp } from './feature/cleanUp.mjs';
import { getConfig, createPath } from './feature/defaultConfig.mjs';
import { updateFromRemote } from './feature/updateFromRemote.mjs';
import { createCatalog } from './util/createCatalog.mjs';
import { debugFunction } from './util/debugFunction.mjs';
import { deletePath } from './util/deletePath.mjs';
import { readFile } from './util/readFile.mjs';
import { updateJson } from './util/updateJson.mjs';

const update = async (args) => {
    const config = await getConfig(args);
    if (!config.fileMap) {
        throw new Error('Config file not exists, use init script');
    }
    debugFunction(config.isDebug, '=== Start SNP UPDATE ===');
    return await createCatalog(config.temporaryFolder).then(() => {
        return { ...config };
    });
};
const cleanUpBeforeUpdate = async (config) => {
    const snpFiles = config.fileMap?.snpFiles;
    for (const fileName in snpFiles) {
        if (snpFiles.hasOwnProperty(fileName)) {
            const snpFile = snpFiles[fileName];
            const snpTypeFile = ['defaultFile', 'instructionsFile'];
            for (const fileType of snpTypeFile) {
                await readFile(snpFile[fileType]);
            }
            await deletePath(createPath([config.projectCatalog, fileName]), config.isDebug);
        }
    }
    await deletePath(config.REPOSITORY_MAP_FILE_NAME, config.isDebug);
    const newConfig = { ...config };
    delete newConfig.fileMap;
    await updateJson({
        filePath: config.snpConfigFile,
        newContent: newConfig,
        replace: true,
    });
    return config;
};
const args = minimist(process.argv.slice(2));
let finalConfig = {
    isDebug: false,
};
update(args)
    .then((config) => {
    finalConfig = config;
    return cleanUpBeforeUpdate(config);
})
    .then((config) => {
    finalConfig = config;
    return updateFromRemote(config);
})
    .then((config) => {
    finalConfig = config;
    return buildFromConfig(config);
})
    .then((config) => {
    finalConfig = config;
    return cleanUp(config);
})
    .finally(() => {
    debugFunction(finalConfig?.isDebug, { finalConfig }, '[INIT] final config');
    debugFunction(finalConfig?.isDebug, '=== final SNP UPDATE ===');
});

export { cleanUpBeforeUpdate, update };
